import _defineProperty from 'babel-runtime/helpers/defineProperty';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _createClass from 'babel-runtime/helpers/createClass';
import _possibleConstructorReturn from 'babel-runtime/helpers/possibleConstructorReturn';
import _inherits from 'babel-runtime/helpers/inherits';
import React from 'react';
import { createPortal } from 'react-dom';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import { dataToArray, transitionEnd } from './utils';

var windowIsUndefined = typeof window === 'undefined';

var Drawer = function (_React$PureComponent) {
  _inherits(Drawer, _React$PureComponent);

  function Drawer(props) {
    _classCallCheck(this, Drawer);

    var _this = _possibleConstructorReturn(this, (Drawer.__proto__ || Object.getPrototypeOf(Drawer)).call(this, props));

    _initialiseProps.call(_this);

    if (props.onIconClick || props.parent || props.iconChild || props.width) {
      console.warn('rc-drawer-menu API has been changed, please look at the releases, ' + 'https://github.com/react-component/drawer-menu/releases');
    }
    _this.state = {
      open: props.open !== undefined ? props.open : !!props.defaultOpen
    };
    return _this;
  }

  _createClass(Drawer, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.getParentAndLevelDom(this.props);
      if (this.props.getContainer || this.props.parent) {
        this.container = this.defaultGetContainer();
      }
      this.forceUpdate();
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      var open = nextProps.open,
          placement = nextProps.placement;

      if (open !== undefined && open !== this.props.open) {
        this.isOpenChange = true;
        this.setState({
          open: open
        });
      }
      if (placement !== this.props.placement) {
        // test 的 bug, 有动画过场，删除 dom
        this.contextDom = null;
      }
      if (this.props.level !== nextProps.level) {
        this.getParentAndLevelDom(nextProps);
      }
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      if (this.container) {
        this.setLevelDomTransform(false, true);
        // 拦不住。。直接删除；
        if (this.props.getContainer) {
          this.container.parentNode.removeChild(this.container);
        }
      }
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      var children = this.getChildToRender();
      if (!this.props.getContainer) {
        return React.createElement(
          'div',
          { className: this.props.wrapperClassName, ref: function ref(c) {
              _this2.container = c;
            } },
          children
        );
      }
      if (!this.container) {
        return null;
      }
      return createPortal(children, this.container);
    }
  }]);

  return Drawer;
}(React.PureComponent);

Drawer.propTypes = {
  wrapperClassName: PropTypes.string,
  open: PropTypes.bool,
  bodyStyle: PropTypes.object,
  defaultOpen: PropTypes.bool,
  placement: PropTypes.string,
  level: PropTypes.oneOfType([PropTypes.string, PropTypes.array]),
  levelTransition: PropTypes.string,
  getContainer: PropTypes.string,
  handleChild: PropTypes.any,
  handleStyle: PropTypes.object,
  onChange: PropTypes.func,
  onMaskClick: PropTypes.func,
  onHandleClick: PropTypes.func,
  showMask: PropTypes.bool,
  maskStyle: PropTypes.object
};
Drawer.defaultProps = {
  className: '',
  prefixCls: 'drawer',
  placement: 'left',
  getContainer: 'body',
  level: 'all',
  levelTransition: 'transform .3s cubic-bezier(0.78, 0.14, 0.15, 0.86)',
  onChange: function onChange() {},
  onMaskClick: function onMaskClick() {},
  onHandleClick: function onHandleClick() {},
  handleChild: React.createElement('i', { className: 'drawer-handle-icon' }),
  handleStyle: {},
  showMask: true,
  maskStyle: {}
};

var _initialiseProps = function _initialiseProps() {
  var _this3 = this;

  this.levelDom = [];
  this.contextDom = null;
  this.maskDom = null;
  this.handleDom = null;
  this.mousePos = null;

  this.getParentAndLevelDom = function (props) {
    if (windowIsUndefined) {
      return;
    }
    var level = props.level,
        getContainer = props.getContainer;

    _this3.levelDom = [];
    _this3.parent = getContainer && document.querySelectorAll(getContainer)[0] || _this3.container.parentNode;
    if (level === 'all') {
      var children = Array.prototype.slice.call(_this3.parent.children);
      children.forEach(function (child) {
        if (child.nodeName !== 'SCRIPT' && child.nodeName !== 'STYLE' && child !== _this3.container) {
          _this3.levelDom.push(child);
        }
      });
    } else if (level) {
      dataToArray(level).forEach(function (key) {
        document.querySelectorAll(key).forEach(function (item) {
          _this3.levelDom.push(item);
        });
      });
    }
  };

  this.trnasitionEnd = function (e) {
    var dom = e.target;
    dom.removeEventListener(transitionEnd, _this3.trnasitionEnd);
    dom.style.transition = '';
  };

  this.onTouchEnd = function (e, close) {
    if (_this3.props.open !== undefined) {
      return;
    }
    var open = close || _this3.state.open;
    _this3.isOpenChange = true;
    _this3.setState({
      open: !open
    });
  };

  this.onMaskTouchEnd = function (e) {
    _this3.props.onMaskClick(e);
    _this3.onTouchEnd(e, true);
  };

  this.onIconTouchEnd = function (e) {
    _this3.props.onHandleClick(e);
    _this3.onTouchEnd(e);
  };

  this.onScrollTouchStart = function (e) {
    if (e.touches.length > 1) {
      return;
    }
    var touchs = e.touches[0];
    _this3.mousePos = {
      x: touchs.pageX,
      y: touchs.pageY
    };
  };

  this.onScrollTouchEnd = function () {
    _this3.mousePos = null;
  };

  this.getScollDom = function (dom) {
    var doms = [];
    var setScrollDom = function setScrollDom(d) {
      if (!d) {
        return;
      }
      if (d.scrollHeight > d.clientHeight || d.scrollWidth > d.clientWidth) {
        doms.push(d);
      }
      if (d !== _this3.contextDom && d !== _this3.handleDom && d !== _this3.maskDom) {
        setScrollDom(d.parentNode);
      }
    };
    setScrollDom(dom);
    return doms[doms.length - 1];
  };

  this.getIsHandleDom = function (dom) {
    if (dom.className === _this3.props.className + '-handle') {
      return true;
    }
    if (dom.parentNode) {
      return _this3.getIsHandleDom(dom.parentNode);
    }
    return false;
  };

  this.removeScroll = function (e) {
    if (!_this3.props.showMask) {
      return;
    }
    var dom = e.target;
    var scrollDom = _this3.getScollDom(dom);
    if (dom === _this3.maskDom || _this3.getIsHandleDom(dom) || !scrollDom) {
      if (e.preventDefault) {
        e.preventDefault();
      }
      e.returnValue = false;
      return;
    }

    var y = e.deltaY;
    var x = e.deltaX;
    if (e.type === 'touchmove') {
      if (e.touches.length > 1 || !_this3.mousePos) {
        return;
      }
      var touches = e.touches[0];
      // 上滑为正，下滑为负
      y = _this3.mousePos.y - touches.pageY;
      x = _this3.mousePos.x - touches.pageX;
    }
    // 竖向
    var scrollTop = scrollDom.scrollTop;
    var height = scrollDom.clientHeight;
    var scrollHeight = scrollDom.scrollHeight;
    var isScrollY = scrollHeight - height > 2;
    var maxOrMinScrollY = isScrollY && (scrollTop <= 0 && y < 0 || scrollTop + height >= scrollHeight && y > 0);
    // 横向
    var width = scrollDom.clientWidth;
    var scrollLeft = scrollDom.scrollLeft;
    var scrollWidth = scrollDom.scrollWidth;
    var isScrollX = scrollWidth - width > 2;
    var maxOrMinScrollX = scrollWidth - width > 2 && (scrollLeft <= 0 && x < 0 || scrollLeft + width >= scrollWidth && x > 0);
    if (!isScrollY && !isScrollX || maxOrMinScrollY || maxOrMinScrollX) {
      if (e.preventDefault) {
        e.preventDefault();
      }
      e.returnValue = false;
      return;
    }
  };

  this.setLevelDomTransform = function (open, openTransition, placementName, value) {
    var _props = _this3.props,
        placement = _props.placement,
        levelTransition = _props.levelTransition,
        onChange = _props.onChange;

    _this3.levelDom.forEach(function (dom) {
      if (_this3.isOpenChange || openTransition) {
        dom.style.transition = levelTransition;
        dom.addEventListener(transitionEnd, _this3.trnasitionEnd);
      }
      var placementPos = placement === 'left' || placement === 'top' ? value : -value;
      dom.style.transform = open ? placementName + '(' + placementPos + 'px)' : '';
    });
    // 处理 body 滚动
    if (!windowIsUndefined) {
      var passiveSupported = false;
      try {
        window.addEventListener('test', null, Object.defineProperty({}, 'passive', {
          get: function get() {
            passiveSupported = true;
          }
        }));
      } catch (err) {
        console.log(err);
      }
      var passive = passiveSupported ? { passive: false } : false;
      if (open) {
        document.body.addEventListener('mousewheel', _this3.removeScroll);
        document.body.addEventListener('touchmove', _this3.removeScroll, passive);
      } else {
        document.body.removeEventListener('mousewheel', _this3.removeScroll);
        document.body.removeEventListener('touchmove', _this3.removeScroll, passive);
      }
    }
    if (onChange && _this3.isOpenChange) {
      onChange(open);
      _this3.isOpenChange = false;
    }
  };

  this.getChildToRender = function () {
    var _classnames;

    var open = _this3.props.open !== undefined ? _this3.props.open : _this3.state.open;
    var _props2 = _this3.props,
        className = _props2.className,
        prefixCls = _props2.prefixCls,
        style = _props2.style,
        placement = _props2.placement,
        children = _props2.children,
        handleChild = _props2.handleChild,
        handleStyle = _props2.handleStyle,
        showMask = _props2.showMask,
        maskStyle = _props2.maskStyle;

    var wrapperClassname = classnames(prefixCls, (_classnames = {}, _defineProperty(_classnames, prefixCls + '-' + placement, true), _defineProperty(_classnames, prefixCls + '-open', open), _defineProperty(_classnames, className, !!className), _classnames));
    var value = _this3.contextDom ? _this3.contextDom.getBoundingClientRect()[placement === 'left' || placement === 'right' ? 'width' : 'height'] : 0;
    var placementName = 'translate' + (placement === 'left' || placement === 'right' ? 'X' : 'Y');
    // 百分比与像素动画不同步，第一次打用后全用像素动画。
    var defaultValue = !_this3.contextDom ? '100%' : value + 'px';
    var placementPos = placement === 'left' || placement === 'top' ? '-' + defaultValue : defaultValue;
    var transform = open ? '' : placementName + '(' + placementPos + ')';
    if (_this3.isOpenChange === undefined || _this3.isOpenChange) {
      _this3.setLevelDomTransform(open, false, placementName, value);
    }
    return React.createElement(
      'div',
      { className: wrapperClassname, style: style },
      showMask && React.createElement('div', {
        className: prefixCls + '-mask',
        onClick: _this3.onMaskTouchEnd,
        style: maskStyle,
        ref: function ref(c) {
          _this3.maskDom = c;
        }
      }),
      React.createElement(
        'div',
        {
          className: prefixCls + '-content-wrapper',
          style: { transform: transform }
        },
        React.createElement(
          'div',
          {
            className: prefixCls + '-content',
            onTouchStart: _this3.onScrollTouchStart,
            onTouchEnd: _this3.onScrollTouchEnd,
            ref: function ref(c) {
              _this3.contextDom = c;
            }
          },
          children
        ),
        handleChild && React.createElement(
          'div',
          {
            className: prefixCls + '-handle',
            onClick: _this3.onIconTouchEnd,
            style: handleStyle,
            ref: function ref(c) {
              _this3.handleDom = c;
            }
          },
          handleChild
        )
      )
    );
  };

  this.defaultGetContainer = function () {
    if (windowIsUndefined) {
      return null;
    }
    var container = document.createElement('div');
    _this3.parent.appendChild(container);
    if (_this3.props.wrapperClassName) {
      container.className = _this3.props.wrapperClassName;
    }
    return container;
  };
};

export default Drawer;